@Echo Off
REM
REM GetCSVFileFtp.cmd, version 1.0.
REM -------------------------------
REM V1.1 - Jesper Wachs, September 2016.
REM
REM CMD file that transfer a file from IBM i to the PC and converts it to CSV format.
REM Further the field descriptions are transferred and used as column headings.
REM
REM The CMD file has 7 parameters:
REM 
REM 1. (Optional) Library containing the file.
REM 2. (Optional) Name of the file.
REM 3. (Optional) Path and name of the PC file. If name or path contains spaces, then
REM               it must be surrounded by ".
REM 4. (Optional) Name or IP address for IBM i.
REM 5. (Optional) Userprofile for IBM i.
REM 6. (Optional) Password for IBM i.
REM 7. (Optional) Write a header line in CSV file.
REM
REM V1.1, September 2016: Added a flag to allow to suppress the header (fields names)
REM                       as it turns out, that the Command intreperter does some mis
REM                       intrepretation of fields that contains special characters
REM                       as '*', ')', '('.
REM
REM V1.0, August 2016: Initial version.
REM
Cls
SetLocal EnableDelayedExpansion
REM
REM Configuration of the CMD.
REM -------------------------
set scriptfile=%~dp0%~n0.script
set logfile=%~dp0%~n0.log
set logfile2=%~dp0%~n02.log
set filelist=%~dp0%~n03.log
set fldhdr=%~dp0%~n04.log
REM
REM Initial error check.
REM --------------------
REM If parm 1 is /? -?, show the help text.
If "%1" == "/?" Goto Help
If "%1" == "-?" Goto Help
Goto AllGood

:Help
REM
REM Show the help text.
REM
Echo.
Echo Syntax of command:
Echo GetCSVFileFtp  library  pf/lf_fil  pc_file  IBM_i  userprofile  password  header
Echo.
Echo Description:
Echo     Transfers a physical or logical file from IBM i the the PC and
Echo     converts data to CSV format using semicolon as field separator and
Echo     comma as decimal point. Further the field descriptions are used as
Echo     coloum headings.
Echo.
Echo Parameters:
Echo     library       Library containing the file to transfer.
Echo.
Echo     pf/lf         Name of the file to transfer.
Echo.
Echo     PC filename   Path and name of the PC file to transfer to.
Echo.
Echo     IBM_i         Name or IP address for the IBM i to transfer from.
Echo.
Echo     userprofile   Userprofile for IBM i.
Echo.
Echo     password      Password for IBM i.
Echo.
Echo     header        A flag to indicate if a header for the CSV file
Echo                   will be written to the file.
Pause
Goto End

:AllGood
REM
REM If parameter Library is missing, then ask for it.
REM
If "%1" == "" (
  Echo.
  set /p lib= Enter the name of the library where the file is located:
  ) Else (
  set lib=%1
  )
REM
REM If parameter File is missing, then ask for it.
REM
If "%2" == "" (
  Echo.
  set /p file= Enter the name of the file containing the data to transfer:
  ) Else (
  set file=%2
  )
REM
REM If parameter PC file is missing, then ask for it.
REM
If "%~3" == "" (
  Echo.
  set /p pcfile= Enter the path and name of the PC file to transfer to:
  ) Else (
  set pcfile=%~3
  )
REM
REM If parameter IBM i is missing, then ask for it.
REM
If "%4" == "" (
  Echo.
  set /p iseriesname= Enter DNS-name or IP address for IBM i:
  ) Else (
  set iseriesname=%4
  )
REM
REM If parameter Userprofile is missing, then ask for it.
REM
If "%5" == "" (
  Echo.
  set /p user= Enter userprofile to use for IBM i:
  ) Else (
  set user=%5
  )
REM
REM If parameter Password is missing, then ask for it.
REM
If "%6" == "" (
  Echo.
  set /p pwd= Enter password for IBM i:
  ) Else (
  set pwd=%6
  )
REM
REM If parameter Password is missing, then ask for it.
REM
If "%7" == "" (
  Echo.
  set /p hdr= Write header to the file [y/n]:
  ) Else (
  set hdr=%6
  )
If /i "%hdr%" NEQ "Y" set hdr=N
Cls
REM
REM Connect and test if userprofile and password is ok, and that hos is actually an IBM i.
REM
Echo.
Echo Checking system, userprofile and password.
Echo.
Echo Please wait ...
Echo.
REM
REM Create FTP script to be executed on IBM i.
REM
Echo open %iseriesname% > "%scriptfile%"
Echo %user% >> "%scriptfile%"
Echo %pwd% >> "%scriptfile%"
Echo quote system >> "%scriptfile%"
Echo Quit >>"%scriptfile%"
REM
REM Run the script.
REM
FTP -s:"%scriptfile%" > "%logfile2%" 2>&1
REM
REM Check if the user got rejected.
REM
FIND "530 " < "%logfile2%" > nul
If not errorlevel == 1 (
Echo.
Call :ErrHdr
Call :ErrDetLin "Userprofile and/or password is not correct for the host."
Call :ErrFtr
Call :ErrFTPLog
Pause
Goto Exit
)
REM
REM If the userprofile was not rejected, then we must be logged on ;-)
REM
FIND "230 " < "%logfile2%" > nul
If errorlevel == 1 (
Echo.
Call :ErrHdr
Call :ErrDetLin "Userprofile could not be logged on the host."
Call :ErrFtr
Call :ErrFTPLog
Pause
Goto Exit
)
REM
REM Check that we are connected to an IBM i.
REM
FIND "OS/400" < "%logfile2%" > nul
If errorlevel == 1 (
Call :ErrHdr
Call :ErrDetLin "The host is not an IBM i."
Call :ErrFtr
Call :ErrFTPLog
Pause
Goto Exit
)
Echo.
Echo Verification successfully done.
Echo.
REM
REM Verify that the PF/LF exists on the IBM i.
REM
Echo.
Echo Verifying that the file %lib%/%file% exists.
Echo.
Echo Please wait ...
Echo.
REM
REM Create FTP script to be executed on the IBM i.
REM
Echo open %iseriesname% > "%scriptfile%"
Echo %user% >> "%scriptfile%"
Echo %pwd% >> "%scriptfile%"
Echo quote rcmd chkobj %lib%/%file% *file aut(*use) >> "%scriptfile%"
Echo Quit >> "%scriptfile%"
REM
REM Run the script.
REM
FTP -s:"%scriptfile%" > "%logfile2%" 2>&1
REM
REM Did an error happen on the CHKOBJ command?
REM
FIND "550-" < "%logfile2%" > nul
If errorlevel == 1 Goto C250
Call :ErrHdr
Call :ErrDetLin "Verification that the file %lib%/%file% exists failed."
Call :ErrFtr
Call :ErrFTPLog
Pause
Goto Exit

:C250
REM
REM The host has returned 250, which is good.
REM
FIND "250" < "%logfile2%" > nul
if errorlevel == 1 (
Call :ErrHdr
Call :ErrDetLin "Verification that the file %lib%/%file% exists failed."
Call :ErrFtr
Call :ErrFTPLog
Pause
Goto Exit
)

:XFer
REM
REM Convert the file to CSV format.
REM
If /i "%hdr%" == "N" Goto XFerFile
Echo.
Echo Preparing the file %lib%/%file%.
Echo.
Echo Please wait ...
Echo.
REM
REM Create FTP script to be executed on the IBM i.
REM
Echo open %iseriesname% >"%scriptfile%"
Echo %user% >>"%scriptfile%"
Echo %pwd% >>"%scriptfile%"
Echo QUOTE TIME 3600 3600 >>"%scriptfile%"
Echo quote rcmd DSPFFD %lib%/%file% OUTPUT(*OUTFILE) OUTFILE(QTEMP/XYZTEMP) >>"%scriptfile%"
Echo quote rcmd CPYTOIMPF FROMFILE(QTEMP/XYZTEMP) TOSTMF('/tmp/%username%') RCDDLM(*CRLF) DTAFMT(*FIXED) >>"%scriptfile%"
Echo quote rcmd DLTF QTEMP/XYZTEMP >>"%scriptfile%"
Echo asc >>"%scriptfile%"
Echo get /tmp/%username% "%filelist%" >>"%scriptfile%"
Echo quote rcmd DEL '/tmp/%username%' >>"%scriptfile%"
Echo Quit >>"%scriptfile%"
REM
REM Run the script.
REM
FTP -s:"%scriptfile%" > "%logfile2%" 2>&1
REM
REM Did an error happen?
REM
FIND "550" < "%logfile2%" > nul
If errorlevel == 1 Goto Cont2
Echo.
Call :ErrHdr
Call :ErrDetLin "Could not retrieve field descriptions from file %lib%/%file%."
Call :ErrFtr
Call :ErrFTPLog
Pause
Goto Exit

:Cont2
if not exist "%filelist%" (
Call :ErrHdr
Call :ErrDetLin "%filelist% was not created. Please check out the log files."
Call :ErrFtr
Call :ErrFTPLog
Pause
Goto Exit
)

:XFerFile
Echo.
Echo Transferring file.
Echo.
Echo Please wait ...
REM
REM Create FTP sript to be executed on the IBM i.
REM
Echo open %iseriesname% >"%scriptfile%"
Echo %user% >>"%scriptfile%"
Echo %pwd% >>"%scriptfile%"
Echo QUOTE TIME 10000 10000 >>"%scriptfile%"
Echo quote rcmd CPYTOIMPF FROMFILE(%lib%/%file%) TOSTMF('/tmp/%username%') RCDDLM(*CRLF) DTAFMT(*DLM) FLDDLM(';') DECPNT(*COMMA) >>"%scriptfile%"
Echo asc >>"%scriptfile%"
Echo get /tmp/%username% "%pcfile%" >>"%scriptfile%"
Echo quote rcmd DEL '/tmp/%username%' >>"%scriptfile%"
Echo Quit >>"%scriptfile%"
REM
REM Run the script.
REM
FTP -s:"%scriptfile%" > "%logfile2%" 2>&1
REM
REM Did an error happen?
REM
FIND "550" < "%logfile2%" > nul
If errorlevel == 1 Goto Cont3
Echo.
Call :ErrHdr
Call :ErrDetLin "Could not transfer the file %lib%/%file%."
Call :ErrFtr
Call :ErrFTPLog
Pause
Goto Exit

:Cont3
If /i "%hdr%" == "N" Goto XFerDone
REM
REM Convert the field file and build heading.
REM
if exist "%fldhdr%" del "%fldhdr%"
for /F "usebackq tokens=*" %%A in ("%filelist%") do (
set line=%%A
set ftext=!line:~184,50!
REM
REM Just a remark: Somethings goes wrong when there is either a asterix or
REM paranthesis in ftext, but we leave that for now.
REM
REM Below a trick to avoid CRLF in the file.
REM
 <NUL set /p= ""!ftext!";" >> "%fldhdr%"
)
Echo.>> "%fldhdr%"
REM
REM Merge the two files.
REM
copy "%fldhdr%"+"%pcfile%" "%fldhdr%"
del "%pcfile%"
REM
REM Get the filename from the full path.
REM
for %%i in ("%pcfile%") do set pcfile=%%~ni%%~xi
rename "%fldhdr%" "%pcfile%"

:XFerDone
Echo.
Echo Transfer is done.
Echo.
REM pause
if exist "%logfile%"  del "%logfile%"
if exist "%logfile2%" del "%logfile2%"
if exist "%filelist%" del "%filelist%"

Goto End

:Exit
REM
REM End when an error has happend.
REM
Echo.
Echo The command was cancelled.
Echo.
Echo A log file has been created and you will find it here:
Echo.
Echo - "%logfile%"
Echo.
Pause

:End
if exist "%scriptfile%" del "%scriptfile%"
if exist "%logfile2%"   del "%logfile2%"
set lib=
set file=
set pcfile=
set filtype=
set iseriesname=
set user=
set pwd=
set scriptfile=
set logfile=
set logfile2=
set filelist=
set fldhdr=
set line=
set ftext=
Goto :eof

REM
REM Subroutines
REM -----------
REM
REM Write Error Header on screen and in log.
REM ----------------------------------------
:ErrHdr
REM
REM In the log file, write the configuration and other information.
REM
Echo %~n0 error log. > "%logfile%"
Echo --------------------- >> "%logfile%"
Echo Date/time ......: %date% %time% >> "%logfile%"
Echo ======================================================================== >> "%logfile%"
Echo IBM i name/IP ..: %iseriesname% >> "%logfile%"
Echo User profile ...: %user% >> "%logfile%"

Call :ErrDetLin "========================================================================"
Call :ErrDetLin "Error occurred."
Call :ErrDetLin "---------------"
Exit /b

REM
REM Write Error Detail line on screen and in log.
REM ---------------------------------------------
:ErrDetLin Str
if "%~1" == "" Echo.
if not "%~1" == "" Echo %~1

if "%~1" == "" Echo. >> "%logfile%"
if not "%~1" == "" Echo %~1 >> "%logfile%"
Exit /b

REM
REM Write Error Footer on screen and in log.
REM ----------------------------------------
:ErrFtr
Call :ErrDetLin "========================================================================"
Exit /b

REM
REM Write FTP error log on screen and in log.
REM -----------------------------------------
:ErrFTPLog
Call :ErrDetLin ""
Call :ErrDetLin "Check the log below:"
Call :ErrDetLin "--------------------------[ Begin of log ]------------------------------"
Type "%logfile2%"
Type "%logfile2%" >> "%logfile%"
Call :ErrDetLin "---------------------------[ End of log ]-------------------------------"
Exit /b
